<?php

namespace Drupal\whatsapp_button\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\File\FileUrlGenerator;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a WhatsApp button block.
 *
 * @Block(
 *   id = "whatsapp_button_block",
 *   admin_label = @Translation("WhatsApp Button"),
 *   category = @Translation("Custom")
 * )
 */
class WhatsappButtonBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Service to obtain module information (path).
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * Service to generate absolute file URLs.
   *
   * @var \Drupal\Core\File\FileUrlGenerator
   */
  protected $fileUrlGenerator;

  /**
   * Service to acces the storeage of files.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $fileStorage;

  /**
   * Constructor with dependency injection.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ModuleExtensionList $module_extension_list,
    FileUrlGenerator $file_url_generator,
    EntityTypeManagerInterface $entity_type_manager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->moduleExtensionList = $module_extension_list;
    $this->fileUrlGenerator = $file_url_generator;

    $this->fileStorage = $entity_type_manager->getStorage('file');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('extension.list.module'),
      $container->get('file_url_generator'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Default configuration.
   */
  public function defaultConfiguration() {
    return [
      'number' => '*********',
      'text_contact' => 'Contact us here',
      'message' => 'Hello, I want to place an order!',
      'image_width' => 40,
      'image_alt' => 'WhatsApp Logo',
      'tooltip' => 'Clicking will open a new tab.',
      'default_image' => '/assets/WhatsApp.svg',
      'position_bottom' => 40,
      'position_right' => 40,
      'desktop_width' => 1024,
      'tablet_width' => 768,
      'text_contact_enabled' => FALSE,
      'image' => [],
      'image_width_tablet_enabled' => FALSE,
      'image_width_tablet' => '',
      'image_width_desktop_enabled' => FALSE,
      'image_width_desktop' => '',
      'position_tablet_enabled' => FALSE,
      'position_bottom_tablet' => '',
      'position_right_tablet' => '',
      'position_desktop_enabled' => FALSE,
      'position_bottom_desktop' => '',
      'position_right_desktop' => '',
      'media_queries_enabled' => FALSE,
    ];
  }

  /**
   * Block form.
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form['number'] = [
      '#type' => 'tel',
      '#title' => $this->t("WhatsApp's phone number"),
      '#description' => $this->t('Add the WhatsApp number, for example: 573053609898.'),
      '#default_value' => $this->configuration['number'],
      '#required' => TRUE,
    ];

    $form['text_detail'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Text information'),
    ];

    $form['text_detail']['message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Message'),
      '#description' => $this->t("Welcome message. By default it says: 'Hello, I want to place an order.'"),
      '#default_value' => $this->configuration['message'],
    ];

    $form['text_detail']['tooltip'] = [
      '#type' => 'textfield',
      '#title' => 'Tooltip',
      '#description' => $this->t('Text displayed when hovering over the link. Used to indicate the action upon clicking. For example: "Clicking will open a new tab."'),
      '#default_value' => $this->configuration['tooltip'],
      '#required' => TRUE,
    ];

    $form['text_detail']['text_contact_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable text aside the icon'),
      '#description' => $this->t('Enable to set a different text for the contact link.<br>By default: "Contact us here"<br>If not enable, only the icon will be visible without text.'),
      '#default_value' => $this->configuration['text_contact_enabled'] || FALSE,
    ];

    $form['text_detail']['text_contact_details'] = [
      '#type' => 'details',
      '#open' => !empty($this->configuration['text_contact']) ? TRUE : FALSE,
      '#title' => $this->t('Text aside the icon'),
      '#states' => [
        'visible' => [
          ':input[name="settings[text_detail][text_contact_enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['text_detail']['text_contact_details']['text_contact'] = [
      '#type' => 'textfield',
      '#title' => 'Contact text',
      '#description' => $this->t('Text to be displayed aside the icon. By default: "Contact us here".'),
      '#default_value' => $this->configuration['text_contact'],
      '#required' => FALSE,
    ];

    $form['image_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t("Image's settings"),
    ];

    $form['image_fieldset']['image'] = [
      '#type' => 'managed_file',
      '#title' => 'Image',
      '#description' => $this->t('Add the image to be used as the button. If no image is uploaded, the WhatsApp logo will appear. Allowed extensions: jpg, jpeg, png, or svg.'),
      '#default_value' => $this->configuration['image'],
      '#upload_location' => 'public://whatsapp_button',
      '#allowed_extensions' => ['jpg jpeg png svg webp avif' => 'jpg jpeg png svg webp avif'],
    ];

    $form['image_fieldset']['image_alt'] = [
      '#type' => 'textfield',
      '#title' => "Image's alternate text",
      '#description' => $this->t('Text to be displayed when the image is not available. By default: "WhatsApp Logo".'),
      '#default_value' => $this->configuration['image_alt'],
      '#required' => TRUE,
    ];

    $form['image_fieldset']['image_width'] = [
      '#type' => 'number',
      '#title' => 'Image width',
      '#description' => $this->t('Assign the image width. The height will scale proportionally.'),
      '#default_value' => $this->configuration['image_width'],
      '#required' => TRUE,
    ];

    $form['image_fieldset']['image_medition_units'] = [
      '#type' => 'select',
      '#title' => $this->t('Image medition units'),
      '#description' => $this->t("Select the units for the image's width values."),
      '#options' => [
        'px' => 'px',
        'rem' => 'rem',
        'em' => 'em',
        '%' => '%',
        'vh' => 'vh',
        'vw' => 'vw',
      ],
      '#default_value' => $this->configuration['image_medition_units'] ?? 'px',
    ];

    $form['image_fieldset']['image_width_tablet_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable tablet image width'),
      '#description' => $this->t('Enable to set a different image width for tablet devices.'),
      '#default_value' => $this->configuration['image_width_tablet_enabled'],
    ];

    $form['image_fieldset']['image_width_tablet'] = [
      '#type' => 'details',
      '#open' => !empty($this->configuration['image_width_tablet']) ? TRUE : FALSE,
      '#title' => $this->t('Tablet image width'),
      '#states' => [
        'visible' => [
          ':input[name="settings[image_fieldset][image_width_tablet_enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['image_fieldset']['image_width_tablet']['image_width_tablet'] = [
      '#type' => 'number',
      '#title' => 'Image width tablet',
      '#description' => $this->t('Assign the image width for tablet devices. The height will scale proportionally.'),
      '#default_value' => $this->configuration['image_width_tablet'],
    ];

    $form['image_fieldset']['image_width_desktop_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable desktop image width'),
      '#description' => $this->t('Enable to set a different image width for desktop devices.'),
      '#default_value' => $this->configuration['image_width_desktop_enabled'],
    ];

    $form['image_fieldset']['image_width_desktop'] = [
      '#type' => 'details',
      '#open' => !empty($this->configuration['image_width_desktop']) ? TRUE : FALSE,
      '#title' => $this->t('Desktop image width'),
      '#states' => [
        'visible' => [
          ':input[name="settings[image_fieldset][image_width_desktop_enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['image_fieldset']['image_width_desktop']['image_width_desktop'] = [
      '#type' => 'number',
      '#title' => 'Image width desktop',
      '#description' => $this->t('Assign the image width for desktop devices. The height will scale proportionally.'),
      '#default_value' => $this->configuration['image_width_desktop'],
    ];

    $form['position_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Position settings'),
    ];

    $form['position_fieldset']['position_bottom'] = [
      '#type' => 'number',
      '#title' => $this->t('Position bottom'),
      '#description' => $this->t('Numeric value to set the bottom position (in pixels or appropriate units).'),
      '#default_value' => $this->configuration['position_bottom'],
      '#required' => TRUE,
    ];

    $form['position_fieldset']['position_right'] = [
      '#type' => 'number',
      '#title' => $this->t('Position right'),
      '#description' => $this->t('Numeric value to set the right position (in pixels or appropriate units).'),
      '#default_value' => $this->configuration['position_right'],
      '#required' => TRUE,
    ];

    $form['position_fieldset']['position_units'] = [
      '#type' => 'select',
      '#title' => $this->t('Position medition units'),
      '#description' => $this->t('Select the units for the position values.'),
      '#options' => [
        'px' => 'px',
        'rem' => 'rem',
        'em' => 'em',
        '%' => '%',
        'vh' => 'vh',
        'vw' => 'vw',
      ],
      '#default_value' => $this->configuration['position_units'] ?? 'px',
    ];

    $form['position_fieldset']['position_tablet_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable tablet position'),
      '#description' => $this->t('Enable to set a different position for tablet devices.'),
      '#default_value' => $this->configuration['position_tablet_enabled'],
    ];

    $form['position_fieldset']['position_tablet'] = [
      '#type' => 'details',
      '#open' => !empty($this->configuration['position_bottom_tablet']) || !empty($this->configuration['position_right_tablet']) ? TRUE : FALSE,
      '#title' => $this->t('Tablet position'),
      '#states' => [
        'visible' => [
          ':input[name="settings[position_fieldset][position_tablet_enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['position_fieldset']['position_tablet']['position_bottom_tablet'] = [
      '#type' => 'number',
      '#title' => $this->t('Position bottom tablet'),
      '#description' => $this->t('Numeric value to set the bottom position for tablet devices (in pixels or appropriate units).'),
      '#default_value' => $this->configuration['position_bottom_tablet'],
    ];

    $form['position_fieldset']['position_tablet']['position_right_tablet'] = [
      '#type' => 'number',
      '#title' => $this->t('Position right tablet'),
      '#description' => $this->t('Numeric value to set the right position for tablet devices (in pixels or appropriate units).'),
      '#default_value' => $this->configuration['position_right_tablet'],
    ];

    $form['position_fieldset']['position_desktop_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable desktop position'),
      '#description' => $this->t('Enable to set a different position for desktop devices.'),
      '#default_value' => $this->configuration['position_desktop_enabled'],
    ];

    $form['position_fieldset']['position_desktop'] = [
      '#type' => 'details',
      '#open' => !empty($this->configuration['position_bottom_desktop']) || !empty($this->configuration['position_right_desktop']) ? TRUE : FALSE,
      '#title' => $this->t('Desktop position'),
      '#states' => [
        'visible' => [
          ':input[name="settings[position_fieldset][position_desktop_enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['position_fieldset']['position_desktop']['position_bottom_desktop'] = [
      '#type' => 'number',
      '#title' => $this->t('Position bottom desktop'),
      '#description' => $this->t('Numeric value to set the bottom position for desktop devices (in pixels or appropriate units).'),
      '#default_value' => $this->configuration['position_bottom_desktop'],
    ];

    $form['position_fieldset']['position_desktop']['position_right_desktop'] = [
      '#type' => 'number',
      '#title' => $this->t('Position right desktop'),
      '#description' => $this->t('Numeric value to set the right position for desktop devices (in pixels or appropriate units).'),
      '#default_value' => $this->configuration['position_right_desktop'],
    ];

    $form['media_queries_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Set media queries width'),
      '#description' => $this->t('Enable to set different widths for desktop, tablet, and mobile devices.'),
      '#default_value' => $this->configuration['media_queries_enabled'],
    ];

    $form['media_queries_fieldset'] = [
      '#type' => 'details',
      '#open' => !empty($this->configuration['tablet_width'] || $this->configuration['desktop_width']) ? TRUE : FALSE,
      '#title' => $this->t('Media queries'),
      '#states' => [
        'visible' => [
          ':input[name="settings[media_queries_enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['media_queries_fieldset']['tablet_width'] = [
      '#type' => 'number',
      '#title' => $this->t('Tablet width'),
      '#description' => $this->t('Numeric value to set the width for tablet devices (in pixels or appropriate units).'),
      '#default_value' => $this->configuration['tablet_width'],
      '#required' => TRUE,
    ];

    $form['media_queries_fieldset']['desktop_width'] = [
      '#type' => 'number',
      '#title' => $this->t('Desktop width'),
      '#description' => $this->t('Numeric value to set the width for desktop devices (in pixels or appropriate units).'),
      '#default_value' => $this->configuration['desktop_width'],
      '#required' => TRUE,
    ];

    $form['media_queries_fieldset']['media_queries_medition_units'] = [
      '#type' => 'select',
      '#title' => $this->t('Media queries medition units'),
      '#description' => $this->t("Select the units for the media queries."),
      '#options' => [
        'px' => 'px',
        'rem' => 'rem',
        'em' => 'em',
        '%' => '%',
        'vh' => 'vh',
        'vw' => 'vw',
      ],
      '#default_value' => $this->configuration['media_queries_medition_units'] ?? 'px',
    ];

    return $form;
  }

  /**
   * Block submission handler.
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['number'] = $form_state->getValue('number');
    $this->configuration['message'] = $form_state->getValue(['text_detail', 'message']);
    $this->configuration['tooltip'] = $form_state->getValue(['text_detail', 'tooltip']);
    $this->configuration['text_contact'] = $form_state->getValue(['text_detail', 'text_contact_details', 'text_contact']);
    $this->configuration['text_contact_enabled'] = $form_state->getValue(['text_detail', 'text_contact_enabled']);

    /* Image */
    $this->configuration['image'] =
    $form_state->getValue(['image_fieldset', 'image']);

    $this->configuration['image_alt'] =
    $form_state->getValue(['image_fieldset', 'image_alt']);

    $this->configuration['image_width'] =
    $form_state->getValue(['image_fieldset', 'image_width']);

    $this->configuration['image_medition_units'] =
    $form_state->getValue(['image_fieldset', 'image_medition_units']);

    /* Image width tablet */
    $this->configuration['image_width_tablet_enabled'] =
    $form_state->getValue(['image_fieldset', 'image_width_tablet_enabled']);

    $this->configuration['image_width_tablet'] =
    $form_state->getValue(['image_fieldset', 'image_width_tablet', 'image_width_tablet']);

    /* Image width desktop */
    $this->configuration['image_width_desktop_enabled'] =
    $form_state->getValue(['image_fieldset', 'image_width_desktop_enabled']);

    $this->configuration['image_width_desktop'] =
    $form_state->getValue(['image_fieldset', 'image_width_desktop', 'image_width_desktop']);

    /* Position general */
    $this->configuration['position_bottom'] =
    $form_state->getValue(['position_fieldset', 'position_bottom']);

    $this->configuration['position_right'] =
    $form_state->getValue(['position_fieldset', 'position_right']);

    /* Position tablet */
    $this->configuration['position_tablet_enabled'] =
    $form_state->getValue(['position_fieldset', 'position_tablet_enabled']);

    $this->configuration['position_bottom_tablet'] =
    $form_state->getValue(['position_fieldset', 'position_tablet', 'position_bottom_tablet']);

    $this->configuration['position_right_tablet'] =
    $form_state->getValue(['position_fieldset', 'position_tablet', 'position_right_tablet']);

    /* Position desktop */
    $this->configuration['position_desktop_enabled'] =
    $form_state->getValue(['position_fieldset', 'position_desktop_enabled']);

    $this->configuration['position_bottom_desktop'] =
    $form_state->getValue(['position_fieldset', 'position_desktop', 'position_bottom_desktop']);

    $this->configuration['position_right_desktop'] =
    $form_state->getValue(['position_fieldset', 'position_desktop', 'position_right_desktop']);

    $this->configuration['position_units'] =
    $form_state->getValue(['position_fieldset', 'position_units']);

    /* Media queries */
    $this->configuration['media_queries_enabled'] =
    $form_state->getValue('media_queries_enabled');

    $this->configuration['tablet_width'] =
    $form_state->getValue(['media_queries_fieldset', 'tablet_width']);

    $this->configuration['desktop_width'] =
    $form_state->getValue(['media_queries_fieldset', 'desktop_width']);

    $this->configuration['media_queries_medition_units'] =
    $form_state->getValue(['media_queries_fieldset', 'media_queries_medition_units']);
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $cardImage = $this->configuration['image'];
    $imgSrc = NULL;

    if (!empty($cardImage[0]) && ($file = $this->fileStorage->load($cardImage[0]))) {
      $imgSrc = $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri());
    }
    else {
      $moduleRelativePath = $this->configuration['default_image'];
      $modulePath = $this->moduleExtensionList->getPath('whatsapp_button');
      $fullPath = $modulePath . '/' . $moduleRelativePath;
      $imgSrc = $this->fileUrlGenerator->generateAbsoluteString($fullPath);
    }

    $build['whatsapp_block'] = [
      '#theme' => 'whatsapp_button_template',
      '#tooltip' => $this->configuration['tooltip'],
      '#message' => $this->configuration['message'],
      '#number' => $this->configuration['number'],
      '#text_contact_enabled' => $this->configuration['text_contact_enabled'],
      '#text_contact' => $this->configuration['text_contact'],
      '#image' => [
        '#theme' => 'image',
        '#src' => $imgSrc ?? $this->configuration['default_image'],
        '#alt' => $this->configuration['image_alt'],
        '#attributes' => [
          'width' => $this->configuration['image_width'],
          'height' => 'auto',
        ],
      ],
      '#image_width_tablet_enabled' => $this->configuration['image_width_tablet_enabled'],
      '#image_width_tablet' => $this->configuration['image_width_tablet'],
      '#image_width_desktop_enabled' => $this->configuration['image_width_desktop_enabled'],
      '#image_width_desktop' => $this->configuration['image_width_desktop'],
      '#image_medition_units' => $this->configuration['image_medition_units'],
      '#bottom' => $this->configuration['position_bottom'],
      '#right' => $this->configuration['position_right'],
      '#position_tablet_enabled' => $this->configuration['position_tablet_enabled'],
      '#position_desktop_enabled' => $this->configuration['position_desktop_enabled'],
      '#bottom_tablet' => $this->configuration['position_bottom_tablet'],
      '#right_tablet' => $this->configuration['position_right_tablet'],
      '#bottom_desktop' => $this->configuration['position_bottom_desktop'],
      '#position_units' => $this->configuration['position_units'],
      '#right_desktop' => $this->configuration['position_right_desktop'],
      '#tablet_width' => $this->configuration['tablet_width'],
      '#desktop_width' => $this->configuration['desktop_width'],
      '#media_queries_medition_units' => $this->configuration['media_queries_medition_units'],
    ];

    return $build;
  }

}
